@@ -20,7 +20,7 @@ class OAuth2CustomStrategy extends Strategy {
 
   userProfile (accessToken, done) {
     this._oauth2.get(this._userProfileURL, accessToken, function (err, body, res) {
-      let json
+      let json, profile
 
       if (err) {
         return done(new InternalOAuthError('Failed to fetch user profile', err))
@@ -33,7 +33,11 @@ class OAuth2CustomStrategy extends Strategy {
       }
 
       checkAuthorization(json, done)
-      const profile = parseProfile(json)
+      try {
+        profile = parseProfile(json)
+      } catch (ex) {
+        return done('Failed to identify user profile information', null)
+      }
       profile.provider = 'oauth2'
 
       done(null, profile)
@@ -58,6 +62,11 @@ function parseProfile (data) {
   const displayName = extractProfileAttribute(data, config.oauth2.userProfileDisplayNameAttr)
   const email = extractProfileAttribute(data, config.oauth2.userProfileEmailAttr)
 
+  if (id === undefined && username === undefined) {
+    logger.error('oauth2 auth failed: id and username are undefined')
+    throw new Error('User ID or Username required')
+  }
+
   return {
     id: id || username,
     username,
@@ -75,7 +84,13 @@ function checkAuthorization (data, done) {
       logger.error('oauth2: "accessRole" is configured, but "rolesClaim" is missing from the config. Can\'t check group membership!')
     } else {
       // parse and check role data
-      const roles = extractProfileAttribute(data, config.oauth2.rolesClaim)
+      let roles = []
+      try {
+        roles = extractProfileAttribute(data, config.oauth2.rolesClaim)
+      } catch (err) {
+        logger.warn(`oauth2: failed to extract rolesClaim '${config.oauth2.rolesClaim}' from user profile.`)
+        return done('Permission denied', null)
+      }
       if (!roles) {
         logger.error('oauth2: "accessRole" is configured, but user profile doesn\'t contain roles attribute. Permission denied')
         return done('Permission denied', null)
@@ -91,7 +106,7 @@ function checkAuthorization (data, done) {
 
 OAuth2CustomStrategy.prototype.userProfile = function (accessToken, done) {
   this._oauth2.get(this._userProfileURL, accessToken, function (err, body, res) {
-    let json
+    let json, profile
 
     if (err) {
       return done(new InternalOAuthError('Failed to fetch user profile', err))
@@ -104,7 +119,11 @@ OAuth2CustomStrategy.prototype.userProfile = function (accessToken, done) {
     }
 
     checkAuthorization(json, done)
-    const profile = parseProfile(json)
+    try {
+      profile = parseProfile(json)
+    } catch (ex) {
+      return done('Failed to identify user profile information', null)
+    }
     profile.provider = 'oauth2'
 
     done(null, profile)
